@@ -97,16 +97,19 @@ class AsyncIoTask(object):
                         print(f'图像数据文件格式存在双后缀：{file_path}  请核对数据')
                     else:  # 处理正常数据，包含重构列表原始封装列表
                         if isinstance(judge_dir, str) or isinstance(index, bool) or isinstance(index, str) or isinstance(index, argparse.Namespace):
-                            save_images_dir = Path(data_info['output_dir'], data_info['image_dir'])
-                            # save_images_dir = os.path.join(data_info['output_dir'], data_info['image_dir'])
-                            os.makedirs(save_images_dir, exist_ok=True)
-                            save_labelme_dir = Path(data_info['output_dir'], data_info['labelme_dir'])
-                            # save_labelme_dir = os.path.join(data_info['output_dir'], data_info['labelme_dir'])
-                            os.makedirs(save_labelme_dir, exist_ok=True)
+                            if custom_label:
+                                save_images_dir = str(Path(data_info['output_dir'], custom_label, data_info['image_dir']))
+                                os.makedirs(save_images_dir, exist_ok=True)
+                                save_labelme_dir = str(Path(data_info['output_dir'], custom_label, data_info['labelme_dir']))
+                                os.makedirs(save_labelme_dir, exist_ok=True)
+                            else:
+                                save_images_dir = Path(data_info['output_dir'], data_info['image_dir'])
+                                os.makedirs(save_images_dir, exist_ok=True)
+                                save_labelme_dir = Path(data_info['output_dir'], data_info['labelme_dir'])
+                                os.makedirs(save_labelme_dir, exist_ok=True)
                             # 图像文件处理，json文件处理
                             if index is True:  # 文件移动
-                                tasks.append(asyncio.create_task(
-                                    self.move_file(data_info['full_path'], save_images_dir)))
+                                tasks.append(asyncio.create_task(self.move_file(data_info['full_path'], save_images_dir)))
                                 if data_info.get('background') is True:  # 不为背景才移动json文件
                                     tasks.append(asyncio.create_task(self.move_file(data_info['original_json_path'], save_labelme_dir)))
                             else:  # 文件拷贝
