@@ -52,6 +52,9 @@ def parser_args():
     parser.add_argument('--sync', action="store_true", help="默认False，填写参数代表为true，用于判断是同步处理，还是异步处理。是为True，否为False")
     parser.add_argument('--threshold', type=float, help="阈值参数，模型预测数据集，设定的阈值")
     parser.add_argument('--leak-check', action="store_true", help="默认False，填写参数代表为true，用于筛选漏检出数据。是为True，否为False")
+    parser.add_argument('--randomize', action="store_true", help="默认False，填写参数代表为true，用于列表随机化。是为True，否为False")
+    parser.add_argument('--test-ratio', type=float, default=0.2, help='默认测试集比例值')
+    parser.add_argument('--train-ratio', type=float, default=0.8, help='默认训练集比例值')
     parser.add_argument('--error-check', action="store_true", help="默认False，填写参数代表为true，用于筛选误检出数据。是为True，否为False")
     parser.add_argument('--right-check', action="store_true", help="默认False，填写参数代表为true，用于筛选正确检出数据。是为True，否为False")
     parser.add_argument('--coco-to-bbox', action="store_true", help="默认False，填写参数代表为true，用于判断多边形转矩形框。是为True，否为False")
@@ -59,6 +62,7 @@ def parser_args():
     parser.add_argument('--is-PCA', action="store_true",
                         help="默认False，填写参数代表为true，用于直方图降维条件判断。是为True，但会造成信息丢失，否为False，信息更准确")
     parser.add_argument('--delete', action="store_true", help="默认False，填写参数代表为true，用于删除勾选数据。是为True，否为False")
+    parser.add_argument('--flags-true', action="store_true", help="默认False，填写参数代表为true，用于删除勾选数据。是为True，否为False")
     parser.add_argument('--opt', type=str, help="opt值为local表示局部直方图计算，opt值为global表示全局直方图计算")
     parser.add_argument('--json-file', type=str, help="图像文件约定目录规则,00.images")
     parser.add_argument('--image-file', type=str, help="json文件约定目录规则，01.labelme")
@@ -153,6 +157,14 @@ def parser_args():
         return args
     elif args.function == 'modify_flags':  # 修改flags字典值
         return args
+    elif args.function == 'filter_flags_true':  # 筛选flags字典值为真的数据
+        return args
+    elif args.function == 'ratio':  # 筛选flags字典值为真的数据
+        return args
+    elif args.function == 'pinyin_images':  # 针对分类图像数据，没有labelme的情况，把文件夹中文转换为拼音
+        return args
+    elif args.function == 'category_name':  # 打印分类类别名称
+        return args
     else:
         assert not args.function, '传入的操作功能参数不对:{}'.format(args.function)
 
@@ -173,8 +185,13 @@ def main():
             print(time.time() - async_time)
         if args.function == 'pinyin':
             data_info.hanzi_to_pinyin()  # 对中文路径转拼音后，重新输出
+        if args.function == 'pinyin_images':
+            data_info.hanzi_to_pinyin_images()  # 对中文路径转拼音后，只处理有图像的情况，没有labelme
         if args.function == 'check_format':  # 检查图像格式
             data_info.check_images_format()
+        if args.function == 'category_name':  # 检查图像格式
+            data_info.print_icategory_name()
+
     else:  # 异步读写数据处理，存在少部分同步写数据处理
         async_time = time.time()
         data_info = LabelmeLoad(args, input_datasets_list)  # 初始化输入参数
@@ -298,6 +315,10 @@ def main():
             dataset.move_file(args)
         elif args.function == 'modify_flags':  # 修改falgs值
             dataset.modify_flags(args)
+        elif args.function == 'filter_flags_true':  # 筛选falgs值为真的标注
+            dataset.select_flags_true(args)
+        elif args.function == 'ratio':  # 按比例划分测试集和训练集
+            dataset.ratio_split(args)
 
 
 if __name__ == '__main__':
