@@ -17,6 +17,8 @@ from tqdm.asyncio import tqdm_asyncio  # 引入 tqdm 的异步版本
 import shutil
 from pypinyin import pinyin, Style
 import zipfile
+
+
 # from googletrans import Translator
 
 
@@ -41,19 +43,91 @@ class LabelmeLoad(object):
 
     async def read_directory(self, root_dir: str) -> List[str]:
         """
-        异步并发读取目录下的所有文件路径
+        异步并发读取目录下的所有图像文件路径，排除json文件
         """
+        # file_paths = []
+        # for entry in os.scandir(root_dir):
+        #     if entry.is_dir():
+        #         if entry.path.endswith('01.labelme'):  # 忽略以 01.labelme结尾的目录
+        #             continue
+        #         sub_paths = await self.read_directory(entry.path)
+        #         file_paths.extend(sub_paths)
+        #     else:
+        #         file_paths.append(entry.path)
+        # return file_paths
         file_paths = []
         for entry in os.scandir(root_dir):
-            if entry.is_dir():
-                if entry.path.endswith('01.labelme'):  # 忽略以 01.labelme结尾的目录
-                    continue
+            if entry.is_file() and entry.name.endswith(tuple(self.parameter.file_formats)):
+                file_paths.append(entry.path)
+            elif entry.is_dir() and not entry.name.endswith('01.labelme'):  # endswith检查字符串结尾的方法
                 sub_paths = await self.read_directory(entry.path)
                 file_paths.extend(sub_paths)
-            else:
-                file_paths.append(entry.path)
         return file_paths
 
+    async def creat_directory(self, root_dir: str) -> List[str]:
+        """
+        获取json和图像路径
+        @param root_dir:
+        @return:
+        """
+        valid_extensions = ['.jpg', '.jpeg', '.png', '.JPEG', '.JPG', '.PNG', '.webp', '.json']
+        # valid_extensions = ['.jpg', '.jpeg', '.png', '.JPEG', '.JPG', '.PNG', '.webp']
+        file_all_paths = []
+        for entry in os.scandir(root_dir):
+            if entry.is_file() and entry.name.endswith(tuple(valid_extensions)):
+                file_all_paths.append(entry.path)
+            elif entry.is_dir():  # endswith检查字符串结尾的方法
+                sub_paths = await self.creat_directory(entry.path)
+                file_all_paths.extend(sub_paths)
+        return file_all_paths
+
+    def linshi_directory(self, root_dir: str) -> List[str]:
+        ok_path = list()
+        for root, dirs, files in tqdm(os.walk(root_dir, topdown=True)):
+            print(root)
+            if root.count('00.images') == 2 or root.count('01.labelme') == 1:  # 设计规则，根据00.images目录，做唯一判断
+                # if path_name not in file_path:
+                if not os.listdir(root):  # 判断目录是否为空，如果目录下不存在文件就删除目录
+                    os.rmdir(root)
+                    # ok_path.append(root)
+            # for file in files:
+            #     path_name = os.path.join(root, file).replace('\\', '/')
+            #     obj_path = Path(file)  # 初始化路径对象为对象
+            #     if obj_path.suffix in self.parameter.file_formats:
+            #         # 所有labelme数据集存放规则为：图像必须存放在00.images目录中，图像对应的json文件必须存放在01.labelme中
+
+        # file_all_paths = []
+        # ok_path = list()
+        # # visited_dirs = set()  # 存储已访问的目录名称
+        # for entry in os.scandir(root_dir):
+        #     # if entry.is_file() and entry.name.endswith(tuple(valid_extensions)):
+        #     #     file_all_paths.append(entry.path)
+        #     if entry.is_dir():
+        #         # if "00.images" in entry.path or "01.labelme" in entry.path:
+        #         print(entry.path)
+        # ok_path.append(entry.path)
+        # if entry.path not in file_all_paths:
+        #     file_all_paths.append(entry.path)
+        # if "00.images" not in entry.name:  # 检查是否已访问过该目录
+        # visited_dirs.add(entry.name)  # 将已访问的目录添加到集合中
+        # sub_paths = await self.linshi_directory(entry.path)
+        # print(sub_paths)
+        # file_all_paths.extend(sub_paths)
+        # file_all_paths.extend(sub_paths)
+        return ok_path
+
+    @staticmethod
+    def has_duplicate_folder_name(path, folder_name):
+        folders = path.split('\\')  # 以'\\'拆分路径名
+        count = 0
+
+        for folder in folders:
+            if folder == folder_name:
+                count += 1
+                if count >= 2:
+                    return True
+        return False
+
     @staticmethod
     async def calculate_file_md5(file_path: str) -> str:
         """
@@ -90,8 +164,16 @@ class LabelmeLoad(object):
         if content is None:
             print(f'图像文件内容为空，请核对该文件路径{file_path}')
             exit()
-        md5_value = await asyncio.get_running_loop().run_in_executor(self._executor, hashlib.md5, content)
-        return md5_value.hexdigest()
+        # md5_value = await asyncio.get_running_loop().run_in_executor(self._executor, hashlib.md5, content)
+        # return md5_value.hexdigest()
+        # 修改MD5算法，使用SHA3-512算法，碰撞抵抗性，提供足够的安全性
+        sha3_512_value = await asyncio.get_running_loop().run_in_executor(self._executor, self.calculate_sha3_512, content)
+        return sha3_512_value
+
+    @staticmethod
+    def calculate_sha3_512(data: bytes) -> str:
+        sha3_512_hash = hashlib.sha3_512(data)
+        return sha3_512_hash.hexdigest()
 
     @staticmethod
     async def load_labelme(data_path: dict) -> dict:
@@ -125,34 +207,42 @@ class LabelmeLoad(object):
     async def process_file(self, file_path: str, root_dir: str) -> Union[dict, str]:  # 返回两种类型之一，要么是一个字典（dict），要么是一个字符串（str）。
         """
         异步处理文件，返回封装后的数据结构
+        针对一次性加载许多数据到内存导致，内存超载问题，方案如下。
+        1、分批处理：将文件分成小批量，逐批处理。您可以将文件分为几个子集，然后逐一处理每个子集。这可以减少内存使用，并允许您逐步完成任务。
+        2、使用数据库：将文件数据导入数据库，然后使用数据库查询来处理数据。数据库可以优化大规模数据的存储和检索。（最优推荐）
         """
         obj_path = Path(file_path)
         if obj_path.suffix in self.parameter.file_formats:
             # 设计规则，根据图片文件查找json文件，同时根据约定的目录规则封装labelme数据集
             if file_path.count('00.images') == 1 or self.parameter.output_format == 'voc' or self.parameter.output_format == 'sys':
-                relative_path = os.path.join('..', obj_path.parent.name, obj_path.name)
-                image_dir = str(obj_path.parent).replace('\\', '/').replace(root_dir, '').strip('\\/')
-                labelme_dir = os.path.join(image_dir.replace('00.images', '').strip('\\/'), '01.labelme')
+                # relative_path = os.path.join('..', obj_path.parent.name, obj_path.name)
+                relative_path = str(Path('..', obj_path.parent.name, obj_path.name))
+                # image_dir = str(obj_path.parent).replace('\\', '/').replace(root_dir, '').strip('\\/')
+                image_dir = obj_path.parent.relative_to(root_dir)
+                labelme_dir = str(Path(image_dir.parent, '01.labelme'))
+                # labelme_dir = os.path.join(image_dir.replace('00.images', '').strip('\\/'), '01.labelme')
                 labelme_file = obj_path.stem + '.json'
                 json_path = None
+                output_dir = str(Path(self.parameter.output_dir))
                 if self.parameter.output_dir:
                     # 打印的时候不需要用到，非打印功能，都会用到
-                    json_path = os.path.join(self.parameter.output_dir, labelme_dir, labelme_file)
-                original_json_path = os.path.join(root_dir, labelme_dir, labelme_file)
+                    json_path = str(Path(self.parameter.output_dir, labelme_dir, labelme_file))
+                original_json_path = str(Path(root_dir, labelme_dir, labelme_file))
                 md5_value = await self.calculate_file_md5_async(file_path)  # 如果不是图像，这里获取MD5值是无法获取的，会直接跳转，存在未知图像后缀格式数据的逻辑
                 if self.parameter.output_dir:  # 如果有输出路径，则自定义错误输出目录
-                    self.group_error_path = os.path.join(self.parameter.output_dir, 'group_error_path')
-                    self.out_of_bounds_path = os.path.join(self.parameter.output_dir, 'out_of_bounds_path')
-                    self.error_path = os.path.join(self.parameter.output_dir, 'error_path')
+                    self.group_error_path = str(Path(self.parameter.output_dir, 'group_error_path'))
+                    self.out_of_bounds_path = str(Path(self.parameter.output_dir, 'out_of_bounds_path'))
+                    self.error_path = str(Path(self.parameter.output_dir, 'error_path'))
                 image = Image.open(file_path)
-                data_path = dict(image_dir=image_dir,  # 封装图像目录相对路径，方便后期路径重组及拼接
+                # image, check = self.is_valid_image(file_path)  # 暂时不用
+                data_path = dict(image_dir=str(image_dir),  # 封装图像目录相对路径，方便后期路径重组及拼接
                                  image_file=obj_path.name,  # 封装图像文件名称
                                  image_width=image.width,  # 封装图像宽度
                                  image_height=image.height,  # 封装图像高度
                                  labelme_dir=labelme_dir,  # 封装json文件相对目录
                                  labelme_file=labelme_file,  # 封装json文件名称
                                  input_dir=root_dir,  # 封装输入路径目录
-                                 output_dir=self.parameter.output_dir,  # 封装输出路径目录
+                                 output_dir=output_dir,  # 封装输出路径目录
                                  group_error_path=self.group_error_path,  # 标注分组出错路径
                                  out_of_bounds_path=self.out_of_bounds_path,  # 标注超出图像边界错误路径
                                  error_path=self.error_path,  # 错误数据存放总目录，不分错误类别
@@ -162,11 +252,12 @@ class LabelmeLoad(object):
                                  data_type=self.type_args[0].get('type'),
                                  labelme_info=None,  # 封装一张图像标注属性信息
                                  background=True,  # 封装一张图像属于负样本还是正样本，默认为True，正样本，有标注
-                                 full_path=file_path,  # 封装一张图像绝对路径
+                                 full_path=str(obj_path),  # 封装一张图像绝对路径
                                  json_path=json_path,  # 封装一张图像对应json文件绝对路径，用于输出时写文件的路径使用
                                  original_json_path=original_json_path,  # 封装原始json文件绝对路径
                                  md5_value=md5_value,  # 封装一张图像MD5值，用于唯一性判断
                                  relative_path=relative_path,
+                                 # check=check,  # 图像校验结果记录，true为合格图像，false为不合格图像
                                  # 封装图像使用标注工具读取相对路径，格式为：..\\00.images\\000000000419.jpg
                                  only_annotation=False, )  # 封装是图像还是处理图像对应标注内容的判断条件，默认图片和注释文件一起处理
                 labelme_info = await self.load_labelme(data_path)  # 异步加载json文件
@@ -184,17 +275,22 @@ class LabelmeLoad(object):
         异步非阻塞并发遍历多级目录
         """
         all_images_file_path = []
-        file_paths = await self.read_directory(root_dir)  # 异步读取文件路径
+
         print("异步读取文件路径完成")
         # tasks = [self.process_file(file_path, root_dir) for file_path in file_paths]  # 列表推导式处理文件异步任务列表，与下面的三行代码区别不大
-        if args == "create_dir":
-            return file_paths
+        if args == "create_dir" or args == "move":
+            file_all_paths = await self.creat_directory(root_dir)  # 异步读取文件路径,包含图像和json路径
+            # file_all_paths = self.linshi_directory(root_dir)  # 临时删除目录方法
+            return file_all_paths
         else:
-            tasks = list()
+            file_paths = await self.read_directory(root_dir)  # 异步读取文件路径，只获取图像路径
+            semaphore = asyncio.Semaphore(80000)  # 限制并发数为80000，根据实际情况调整
+            tasks = [self.process_file_with_semaphore(file_path, root_dir, semaphore) for file_path in tqdm(file_paths)]
+            # tasks = list()
             print("异步封装数据开始")
-            for file_path in tqdm(file_paths):
-                # 使用 ensure_future() 函数能够确保协程一定被封装成任务对象，即使协程返回的是普通值而不是协程对象，该值也会被封装为一个 Future 对象，成为可调度的任务。
-                tasks.append(asyncio.ensure_future(self.process_file(file_path, root_dir)))
+            # for file_path in tqdm(file_paths):
+            #     # 使用 ensure_future() 函数能够确保协程一定被封装成任务对象，即使协程返回的是普通值而不是协程对象，该值也会被封装为一个 Future 对象，成为可调度的任务。
+            #     tasks.append(asyncio.ensure_future(self.process_file(file_path, root_dir)))
             # 使用 asyncio.gather() 函数同时运行多个异步任务
             results = await asyncio.gather(*tasks)
             # 使用 tqdm.asyncio.tqdm() 上下文管理器将异步任务的执行过程打印到进度条中
@@ -205,6 +301,9 @@ class LabelmeLoad(object):
                     progress_bar.update(1)
             return all_images_file_path
 
+    async def process_file_with_semaphore(self, file_path: str, root_dir: str, semaphore: asyncio.Semaphore) -> Union[dict, str]:
+        async with semaphore:
+            return await self.process_file(file_path, root_dir)
         # tasks = [self.process_file(file_path, root_dir) for file_path in
         #          file_paths]  # 处理文件异步任务列表
         # results = await asyncio.gather(*tasks)  # 并发处理文件
@@ -282,6 +381,26 @@ class LabelmeLoad(object):
             except Exception as e:
                 print(f"拷贝 {rename_dir} 失败: {e}")
 
+    def hanzi_to_pinyin_images(self):
+        """
+        重命名目录汉字为拼音
+        """
+        for root, dirs, files in tqdm(os.walk(self.type_args[0].get('input_dir'), topdown=True)):
+            for dirname in dirs:
+                original_dir = os.path.join(root, dirname)
+                pinyin_dirname = self.convert_chinese_to_pinyin(dirname)
+                new_dir = os.path.join(root, pinyin_dirname)
+                try:
+                    os.rename(original_dir, new_dir)
+                except Exception as e:
+                    print(f"重命名，存在相同名称的拼音 {original_dir} 失败: {e}")
+            # # 递归处理子目录
+            for dirpath, dirnames, filenames in os.walk(self.type_args[0].get('input_dir')):
+                for subdir in dirnames:
+                    self.convert_chinese_to_pinyin(os.path.join(dirpath, subdir))
+                # 重命名路径
+        print(f'注意不嵌套重命名自目录，嵌套几次目录就多执行几次指令')
+
     @staticmethod
     def convert_path_to_pinyin(path):
         """
@@ -296,6 +415,21 @@ class LabelmeLoad(object):
         new_path = os.path.join(pinyin_path, filename)
         return new_path
 
+    @staticmethod
+    def convert_chinese_to_pinyin(chinese_text):
+        """
+        重命名目录的汉字为拼音
+        @param chinese_text:
+        @return:
+        """
+        pinyin_text = []
+        for char in chinese_text:
+            if isinstance(char, str):
+                pinyin_list = pinyin(char, style=Style.NORMAL)
+                pinyin_path = ''.join([py[0] for py in pinyin_list])  # 提取每个汉字的首字母拼接成新的路径
+                pinyin_text.append(pinyin_path)
+        return ''.join(pinyin_text)
+
     @classmethod
     def get_videos_path(cls, root_dir, file_formats):
         """
@@ -347,3 +481,43 @@ class LabelmeLoad(object):
                         if path_name not in file_name:
                             file_name.append(path_name)
         print(file_name)
+
+    @staticmethod
+    def is_valid_image(file_path, check=True):
+        """
+        验证图像格式及图像通道
+        @param check: 默认为合格图像
+        @param file_path:
+        @return:
+        """
+        try:
+            image = Image.open(file_path)
+            # 验证图像格式是否为TIFF,# 验证图像是否为RGB格式
+            if image.format == "TIFF" or image.mode == "RGB":
+                print("图像校验存在问题" + file_path)
+                return image, False  # 代表不符合标准的图像
+            else:
+                return image, check  # 代表正常符合标准的图像
+        except Exception as e:
+            print(e)
+            print("图像校验存在未知情况问题" + file_path)
+            # 捕获任何异常，包括文件格式不正确或文件损坏
+
+    def check_images_format(self):
+        """
+        找出图像格式后缀
+        """
+        other_format = list()
+        for root, dirs, files in tqdm(os.walk(self.type_args[0].get('input_dir'), topdown=True)):
+            for file in files:
+                path_name = os.path.join(root, file).replace('\\', '/')
+                obj_path = Path(file)  # 初始化路径对象为对象
+                if obj_path.suffix not in self.parameter.file_formats and obj_path.suffix != '.json':
+                    other_format.append(obj_path.suffix)
+                    print(path_name)
+        print(list(set(other_format)))
+
+    def print_icategory_name(self):
+        for root, dirs, files in tqdm(os.walk(self.type_args[0].get('input_dir'), topdown=True)):
+            print(dirs)
+            break
